@@ -1,4 +1,5 @@
 pub mod os_input_output;
+pub mod output;
 pub mod panes;
 pub mod tab;
 
@@ -11,12 +12,13 @@ mod ui;
 mod wasm_vm;
 
 use log::info;
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::{
     path::PathBuf,
     sync::{Arc, Mutex, RwLock},
     thread,
 };
+use zellij_utils::envs;
 use zellij_utils::nix::sys::stat::{umask, Mode};
 use zellij_utils::pane_size::Size;
 use zellij_utils::zellij_tile;
@@ -28,7 +30,6 @@ use crate::{
     os_input_output::ServerOsApi,
     pty::{pty_thread_main, Pty, PtyInstruction},
     screen::{screen_thread_main, ScreenInstruction},
-    tab::Output,
     thread_bus::{Bus, ThreadSenders},
     wasm_vm::{wasm_thread_main, PluginInstruction},
 };
@@ -36,6 +37,7 @@ use route::route_thread_main;
 use zellij_utils::{
     channels::{self, ChannelWithContext, SenderWithContext},
     cli::CliArgs,
+    consts::{DEFAULT_SCROLL_BUFFER_SIZE, SCROLL_BUFFER_SIZE},
     errors::{ContextType, ErrorInstruction, ServerContext},
     input::{
         command::{RunCommand, TerminalAction},
@@ -48,20 +50,20 @@ use zellij_utils::{
     setup::get_default_data_dir,
 };
 
-pub(crate) type ClientId = u16;
+pub type ClientId = u16;
 
 /// Instructions related to server-side application
 #[derive(Debug, Clone)]
-pub(crate) enum ServerInstruction {
+pub enum ServerInstruction {
     NewClient(
         ClientAttributes,
         Box<CliArgs>,
         Box<Options>,
-        LayoutFromYaml,
+        Box<LayoutFromYaml>,
         ClientId,
         Option<PluginsConfig>,
     ),
-    Render(Option<Output>),
+    Render(Option<HashMap<ClientId, String>>),
     UnblockInputThread,
     ClientExit(ClientId),
     RemoveClient(ClientId),
@@ -69,6 +71,7 @@ pub(crate) enum ServerInstruction {
     KillSession,
     DetachSession(ClientId),
     AttachClient(ClientAttributes, Options, ClientId),
+    ConnStatus(ClientId),
 }
 
 impl From<&ServerInstruction> for ServerContext {
@@ -83,6 +86,7 @@ impl From<&ServerInstruction> for ServerContext {
             ServerInstruction::KillSession => ServerContext::KillSession,
             ServerInstruction::DetachSession(..) => ServerContext::DetachSession,
             ServerInstruction::AttachClient(..) => ServerContext::AttachClient,
+            ServerInstruction::ConnStatus(..) => ServerContext::ConnStatus,
         }
     }
 }
@@ -133,9 +137,15 @@ impl SessionState {
         }
     }
     pub fn new_client(&mut self) -> ClientId {
-        let mut clients: Vec<ClientId> = self.clients.keys().copied().collect();
-        clients.sort_unstable();
-        let next_client_id = clients.last().unwrap_or(&0) + 1;
+        let clients: HashSet<ClientId> = self.clients.keys().copied().collect();
+        let mut next_client_id = 1;
+        loop {
+            if clients.contains(&next_client_id) {
+                next_client_id += 1;
+            } else {
+                break;
+            }
+        }
         self.clients.insert(next_client_id, None);
         next_client_id
     }
@@ -186,7 +196,7 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
         .start()
         .expect("could not daemonize the server process");
 
-    std::env::set_var(&"ZELLIJ", "0");
+    envs::set_zellij("0".to_string());
 
     let (to_server, server_receiver): ChannelWithContext<ServerInstruction> = channels::bounded(50);
     let to_server = SenderWithContext::new(to_server);
@@ -305,12 +315,36 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 };
 
                 if !&layout.tabs.is_empty() {
-                    for tab_layout in layout.tabs {
+                    for tab_layout in layout.clone().tabs {
                         spawn_tabs(Some(tab_layout.clone()));
                     }
+
+                    let focused_tab = layout
+                        .tabs
+                        .into_iter()
+                        .enumerate()
+                        .find(|(_, tab_layout)| tab_layout.focus.unwrap_or(false));
+                    if let Some((tab_index, _)) = focused_tab {
+                        session_data
+                            .read()
+                            .unwrap()
+                            .as_ref()
+                            .unwrap()
+                            .senders
+                            .send_to_pty(PtyInstruction::GoToTab((tab_index + 1) as u32, client_id))
+                            .unwrap();
+                    }
                 } else {
                     spawn_tabs(None);
                 }
+                session_data
+                    .read()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_plugin(PluginInstruction::AddClient(client_id))
+                    .unwrap();
             }
             ServerInstruction::AttachClient(attrs, options, client_id) => {
                 let rlock = session_data.read().unwrap();
@@ -332,6 +366,10 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .senders
                     .send_to_screen(ScreenInstruction::AddClient(client_id))
                     .unwrap();
+                session_data
+                    .senders
+                    .send_to_plugin(PluginInstruction::AddClient(client_id))
+                    .unwrap();
                 let default_mode = options.default_mode.unwrap_or_default();
                 let mode_info =
                     get_mode_info(default_mode, attrs.palette, session_data.capabilities);
@@ -344,12 +382,11 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .senders
                     .send_to_plugin(PluginInstruction::Update(
                         None,
+                        Some(client_id),
                         Event::ModeUpdate(mode_info),
                     ))
                     .unwrap();
-                for client_id in session_state.read().unwrap().clients.keys() {
-                    os_input.send_to_client(*client_id, ServerToClientMsg::SwitchToMode(mode));
-                }
+                os_input.send_to_client(client_id, ServerToClientMsg::SwitchToMode(mode));
             }
             ServerInstruction::UnblockInputThread => {
                 for client_id in session_state.read().unwrap().clients.keys() {
@@ -368,17 +405,23 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .senders
                         .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                         .unwrap();
-                    // we only do this inside this if because it means there are still connected
-                    // clients
-                    session_data
-                        .write()
-                        .unwrap()
-                        .as_ref()
-                        .unwrap()
-                        .senders
-                        .send_to_screen(ScreenInstruction::RemoveClient(client_id))
-                        .unwrap();
                 }
+                session_data
+                    .write()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_screen(ScreenInstruction::RemoveClient(client_id))
+                    .unwrap();
+                session_data
+                    .write()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_plugin(PluginInstruction::RemoveClient(client_id))
+                    .unwrap();
                 if session_state.read().unwrap().clients.is_empty() {
                     *session_data.write().unwrap() = None;
                     break;
@@ -395,17 +438,23 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .senders
                         .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                         .unwrap();
-                    // we only do this inside this if because it means there are still connected
-                    // clients
-                    session_data
-                        .write()
-                        .unwrap()
-                        .as_ref()
-                        .unwrap()
-                        .senders
-                        .send_to_screen(ScreenInstruction::RemoveClient(client_id))
-                        .unwrap();
                 }
+                session_data
+                    .write()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_screen(ScreenInstruction::RemoveClient(client_id))
+                    .unwrap();
+                session_data
+                    .write()
+                    .unwrap()
+                    .as_ref()
+                    .unwrap()
+                    .senders
+                    .send_to_plugin(PluginInstruction::RemoveClient(client_id))
+                    .unwrap();
             }
             ServerInstruction::KillSession => {
                 let client_ids = session_state.read().unwrap().client_ids();
@@ -427,27 +476,14 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .senders
                         .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                         .unwrap();
-                    // we only do this inside this if because it means there are still connected
-                    // clients
-                    session_data
-                        .write()
-                        .unwrap()
-                        .as_ref()
-                        .unwrap()
-                        .senders
-                        .send_to_screen(ScreenInstruction::RemoveClient(client_id))
-                        .unwrap();
                 }
             }
-            ServerInstruction::Render(mut output) => {
+            ServerInstruction::Render(serialized_output) => {
                 let client_ids = session_state.read().unwrap().client_ids();
-                // Here the output is of the type Option<String> sent by screen thread.
                 // If `Some(_)`- unwrap it and forward it to the clients to render.
                 // If `None`- Send an exit instruction. This is the case when a user closes the last Tab/Pane.
-                if let Some(op) = output.as_mut() {
-                    for (client_id, client_render_instruction) in
-                        op.client_render_instructions.iter_mut()
-                    {
+                if let Some(output) = &serialized_output {
+                    for (client_id, client_render_instruction) in output.iter() {
                         os_input.send_to_client(
                             *client_id,
                             ServerToClientMsg::Render(client_render_instruction.clone()),
@@ -473,6 +509,10 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 }
                 break;
             }
+            ServerInstruction::ConnStatus(client_id) => {
+                os_input.send_to_client(client_id, ServerToClientMsg::Connected);
+                remove_client!(client_id, os_input, session_state);
+            }
         }
     }
 
@@ -490,7 +530,7 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
 pub struct SessionOptions {
     pub opts: Box<CliArgs>,
     pub config_options: Box<Options>,
-    pub layout: LayoutFromYaml,
+    pub layout: Box<LayoutFromYaml>,
     pub plugins: Option<PluginsConfig>,
 }
 
@@ -506,6 +546,15 @@ fn init_session(
         layout,
         plugins,
     } = options;
+
+    SCROLL_BUFFER_SIZE
+        .set(
+            config_options
+                .scroll_buffer_size
+                .unwrap_or(DEFAULT_SCROLL_BUFFER_SIZE),
+        )
+        .unwrap();
+
     let (to_screen, screen_receiver): ChannelWithContext<ScreenInstruction> = channels::unbounded();
     let to_screen = SenderWithContext::new(to_screen);
 
@@ -522,7 +571,7 @@ fn init_session(
     let data_dir = opts.data_dir.unwrap_or_else(get_default_data_dir);
 
     let capabilities = PluginCapabilities {
-        arrow_fonts: config_options.simplified_ui,
+        arrow_fonts: config_options.simplified_ui.unwrap_or_default(),
     };
 
     let default_shell = config_options.default_shell.clone().map(|command| {
